import { isFunction, isEmpty } from "@agile-code/test";
export default class eventBus {
  constructor() {
    this.callbacks = {};
  }
  /**
   * @description 绑定事件
   * @param {*} eventName 事件名
   * @param {*} fn 事件的回调函数
   */
  on(eventName, fn) {
    if (!isFunction(fn)) {
      throw new Error(`自定义事件${eventName}的回调函数类型不正确`);
    }
    // 获取callbacks中eventName的事件回调数组
    let callbacks = this.callbacks[eventName] || [];
    // 将回调函数插入到数组中
    callbacks.push(fn);
    // 将callbacks重新赋值给this.callbacks的eventName
    this.callbacks[eventName] = callbacks;
  }
  /**
   * @description 触发事件
   * @param {*} eventName 事件名
   * @param  {...any} args 触发事件携带的附加参数,可以多个值
   */
  emit(eventName, ...args) {
    if (this.callbacks[eventName] && this.callbacks[eventName].length) {
      this.callbacks[eventName].forEach((fn) => {
        fn && fn.call(null, ...args);
      });
    }
  }
  /**
   * @description
   * 如果没有提供参数，则移除所有的事件监听器；
   * 如果只提供了事件，则移除该事件所有的监听器；
   * 如果同时提供了事件与回调，则只移除这个回调的监听器；
   * @param {*} eventName 事件名
   * @param {*} fn 事件的回调函数
   */
  off(eventName, fn) {
    // 如果没有提供参数，则移除所有的事件监听器
    if (isEmpty(eventName)) return (this.callbacks = {});
    // 如果没有提供回调函数，则移除所有的eventName事件
    if (isEmpty(fn)) return delete this.callbacks[eventName];
    // 如果同时提供了事件与回调，则只移除这个回调的监听器
    this.callbacks[eventName] &&
      this.callbacks[eventName].forEach((itemfn, index) => {
        if (itemfn === fn) {
          this.callbacks[eventName].splice(index, 1);
        }
      });
  }
  /**
   * @description 绑定一次事件
   * @param {*} eventName 事件名
   * @param {*} fn 事件的回调函数
   */
  once(eventName, fn) {
    let self = this;
    let wrapFun = function (...args) {
      fn(...args);
      self.off(eventName, wrapFun);
    };
    this.on(eventName, wrapFun);
  }
}
